#ifdef GL_ES
	precision highp float;
#endif


/// <summary>
/// Uniform variables.
/// <summary>
uniform vec2 ImageSize;
uniform vec2 TexelSize;
uniform float Time;
uniform bool IsMouseDown;
uniform vec2 MousePos;
								// Pass 1: Input = None, Output = FBO Sample0
uniform sampler2D Sample0;		// Pass 2: Output = FBO Sample1
uniform sampler2D Sample1;		// Pass 3: Output = FBO Sample2
uniform sampler2D Sample2;		// Pass 4: Output = FBO Sample3
uniform sampler2D Sample3;		// Pass 5: Output = FBO Sample4
uniform sampler2D Sample4;		// Pass 6: Output = FBO Sample5
//uniform sampler2D Sample5;	// Pass 7: Output = FBO Sample6...


/// <summary>
/// Varying variables.
/// <summary>
varying vec2 vUv;

const float cFov = 60.0 * 3.141592 / 180.0;
const vec4 cSphere = vec4(0, 0, 10, 2);

struct ray
{
	vec3 origin;
	vec3 direction;
};
bool raySphere(in ray r, in vec4 s, out float dist)
{
	vec3 L = s.xyz - r.origin;
	float tca = dot(L, r.direction);

	if(tca < 0.0)
	{
		return false;
	}

	float d = tca * tca + s.w * s.w - dot(L, L);

	dist = tca - sqrt(d);

	if(d < 0.0)
	{
		return false;
	}

	return true;
}
/// <summary>
/// Fragment shader entry.
/// <summary>
void main ()
{
	ray r;
	r.origin = vec3(0, 0, 0);
	r.direction = normalize(vec3(vUv.xy * 2.0 - 1.0, 1.0 / tan(cFov * 0.5)));
	float d;
	if (raySphere(r,cSphere,d))
	{
		vec3 normal = normalize((r.origin + r.direction * d) - cSphere.xyz);
		vec3 l = normalize(normalize(vec3(1000.0, 0.0, -1000.0))-normalize((r.origin + r.direction * d)));
		
		float diffuse = max(dot(normal,l), 0.0);
		float x = (normal.x+1.0)*0.5;
		x += Time * 0.001;
		x = fract(x);
		vec2 spos = vec2(x,(normal.y+1.0)*0.5);
		float h = texture2D(Sample1,spos).x;
		float dummy = texture2D(Sample0,vUv).x;
		h += (dummy*0.01);
		float clouds = texture2D(Sample4,spos).x;
		vec4 result = texture2D(Sample2,vec2(0.5,h));
		
		result=mix(result,vec4(1.0,1.0,1.0,1.0),clouds);
		
		gl_FragColor = mix(vec4(0.0,0.0,0.0,1.0),result,diffuse);
		return;
	}
	gl_FragColor=texture2D(Sample3,vUv);
}